
'''
CloudControl VM related commands
'''

from cccli.exception import *
from sjrpc.core.exceptions import *
from cccli.printer import Printer, color
from cccli.command import TqlCommand


class VMCommand(TqlCommand):
    """Base class for commands related to VMs."""

    TQL_APPEND = ""
    ERROR_MSG = "No VM selected by TQL"

    def __init__(self, cli, argv0):
        TqlCommand.__init__(self, cli, argv0)
        self.tql_filter += self.TQL_APPEND

    @property
    def cmd_name(self):
        # all VM commands have Command_ prefix
        return self.__class__.__name__[len("Command_"):]

    def _do_rpc_call(self):
            self.rpccall(self.cmd_name, self.args[0])

    def __call__(self, argv):
        self.parse_args(argv)
        if len(self.args) != 1:
            raise cmdBadArgument()

        try:
            self._do_rpc_call()
        except cmdError:
            # We catch and re throw the same error to provide with a more
            # contextual error message
            if self.ERROR_MSG is None:
                raise
            else:
                raise cmdError(self.ERROR_MSG)

    def remote_functions(self):
        return {self.cmd_name}


class Command_start(VMCommand):
    '''Start a stopped vm'''

    TQL_APPEND = "&r=vm&status=stopped"
    ERROR_MSG = VMCommand.ERROR_MSG + ", are your VMs stopped ?"


class Command_stop(VMCommand):
    '''Stop a running vm'''

    TQL_APPEND = "&r=vm&status=running"
    ERROR_MSG = VMCommand.ERROR_MSG + ", are your VMs running ?"


class Command_destroy(VMCommand):
    '''Force a vm to stop'''

    TQL_APPEND = "&r=vm&status!=stopped"
    ERROR_MSG = VMCommand.ERROR_MSG + ", are your VMs running ?"


class Command_pause(VMCommand):
    '''Pause a running vm'''

    TQL_APPEND = "&r=vm&status=running"
    ERROR_MSG = VMCommand.ERROR_MSG + ", are your VMs running ?"


class Command_resume(VMCommand):
    '''Resume a paused vm'''

    TQL_APPEND = "&r=vm&status=paused"
    ERROR_MSG = VMCommand.ERROR_MSG + ", are your VMs paused ?"


class Command_disablevirtiocache(VMCommand):
    '''Set virtio cache to none on VMs disk devices'''

    TQL_APPEND = "&r=vm&status=stopped"
    ERROR_MSG = VMCommand.ERROR_MSG + ", are your VMs stopped ?"


class Command_autostart(VMCommand):
    '''Set autostart flag on VMs'''

    TQL_APPEND = "&r=vm"

    def __init__(self, *args, **kwargs):
        VMCommand.__init__(self, *args, **kwargs)
        self.add_option("-D", action="store_false", dest="flag", help="Disable"
                        " autostart")
        self.add_option("-e", default=True, action="store_true", dest="flag",
                       help="Enable autostart")

    def _do_rpc_call(self):
        self.rpccall("autostart", self.args[0], self.options.flag)


class Command_undefine(VMCommand):
    '''Undefine a stopped vm'''

    TQL_APPEND = "&r=vm&status=stopped"
    ERROR_MSG = VMCommand.ERROR_MSG + ", are you VMs stopped ?"

    def __init__(self, *args, **kwargs):
        VMCommand.__init__(self, *args, **kwargs)
        self.add_option("-k", "--keep-storage", action="store_false", dest="clean", default=True,
                        help="Don't remove vm storage")

    def _do_rpc_call(self):
        self.rpccall("undefine", self.args[0], self.options.clean)


class Command_clone(TqlCommand):
    '''Clone vm'''

    def __init__(self, cli, argv0):
        TqlCommand.__init__(self, cli, argv0)
        self.set_usage("%prog [options] <source tql> <dest tql> <name>")

    def __call__(self, argv):
        # Parse argline
        self.parse_args(argv)
        # Check args count
        if len(self.args) != 3:
            raise cmdBadArgument()
        stql = self.args[0]
        dtql = self.args[1]
        newname = self.args[2]
        if self.options.direct:
            self.rpccall("clone", stql, dtql, newname)
        else:
            # list stql
            self.printer.out("<= Source VM:")
            if not self.options.raw:
                self.tql_filter = "&r=vm$status"
            o = self.rpccall("list", stql, _direct=True, _status=True)
            if len(o["objects"]) != 1:
                raise cmdError("You must select only one VM")
            # list dtql
            self.printer.out("=> Destination HV:")
            if not self.options.raw:
                self.tql_filter = "&r=hv$nvm"
            o = self.rpccall("list", dtql, _direct=True, _status=True)
            if len(o["objects"]) != 1:
                raise cmdError("You must select only one HV")
            # check if destination vm does not exist
            o = self.rpccall("list", "%s&h:%s"%(dtql,newname), _direct=True, _status=False)
            if not self.options.raw:
                self.tql_filter = "&r=vm"
            if len(o["objects"]) != 0:
                raise cmdError("Destination VM exist on HV")
            # ask confirmation
            if self.printer.ask("Do you confirm? (I'm god) ") != "I'm god":
                raise cmdWarning("User resign")
            # run migration
            self.tql_filter = ""
            self.rpccall("clone", stql, dtql, newname, _direct=True)

    def remote_functions(self):
        return set(("clone",))
